require "liquid/autoescape"

describe "{% autoescape %}" do

  def verify_template_output(template, expected, context={})
    rendered = Liquid::Template.parse(template).render!(context)
    expect(rendered).to eq(expected)
  end

  it "handles empty content" do
    verify_template_output(
      "{% autoescape %}{% endautoescape %}",
      ""
    )
  end

  it "handles non-variable content" do
    verify_template_output(
      "{% autoescape %}Static{% endautoescape %}",
      "Static"
    )
  end

  it "escapes all dangerous HTML characters" do
    verify_template_output(
      "{% autoescape %}{{ variable }}{% endautoescape %}",
      "&lt;tag&gt; &amp; &quot;quote&quot;",
      "variable" => '<tag> & "quote"'
    )
  end

  it "applies HTML escaping to all variables inside the block tag" do
    verify_template_output(
      "{% autoescape %}{{ one }} {{ two }} {{ three }}{% endautoescape %}",
      "&lt;tag&gt; &amp; &quot;quote&quot;",
      "one" => "<tag>", "two" => "&", "three" => '"quote"'
    )
  end

  it "applies HTML escaping to a filtered variable" do
    verify_template_output(
      "{% autoescape %}{{ filtered | downcase | capitalize }}{% endautoescape %}",
      "A &lt;strong&gt; tag",
      "filtered" => "A <STRONG> Tag"
    )
  end

  it "does not double-escape variables" do
    verify_template_output(
      "{% autoescape %}{{ escaped | escape }}{% endautoescape %}",
      "HTML &amp; CSS",
      "escaped" => "HTML & CSS"
    )
  end

  it "does not escape variables outside the block tag" do
    verify_template_output(
      "{{ variable }} {% autoescape %}{{ variable }}{% endautoescape %} {{ variable }}",
      "& &amp; &",
      "variable" => "&"
    )
  end

  it "can be called multiple times" do
    verify_template_output(
      "{% autoescape %}{{ var }}{% endautoescape %} {{ var }} {% autoescape %}{{ var }}{% endautoescape %}",
      "&amp; & &amp;",
      "var" => "&"
    )
  end

  it "does not escape variables with the skip_escape filter applied" do
    verify_template_output(
      "{% autoescape %}{{ variable | skip_escape }}{% endautoescape %}",
      "<strong>&amp;</strong>",
      "variable" => "<strong>&amp;</strong>"
    )
  end

  it "supports explicit enabling" do
    verify_template_output(
      "{% autoescape true %}{{ variable }}{% endautoescape %}",
      "&amp;",
      "variable" => "&"
    )
  end

  it "supports explicit disabling" do
    verify_template_output(
      "{% autoescape false %}{{ variable }}{% endautoescape %}",
      "&",
      "variable" => "&"
    )
  end

  it "supports nested auto-escaping contexts" do
    verify_template_output(
      "{% autoescape true %}{{ variable }}{% autoescape false %}{{ variable }}{% autoescape true %}{{ variable }}{% endautoescape %}{% endautoescape %}{% endautoescape %}",
      "&amp;&&amp;",
      "variable" => "&"
    )
  end

  it "supports reading the auto-escaping state from a variable" do
    verify_template_output(
      "{% autoescape escape %}{{ variable }}{% endautoescape %}",
      "&amp;",
      "escape" => true,
      "variable" => "&"
    )
  end

  it "raises an error when called with multiple arguments" do
    invalid = "{% autoescape one two %}{% endautoescape %}"
    expect { Liquid::Template.parse(invalid) }.to raise_error(Liquid::SyntaxError)
  end

  it "does not escape captured variables" do
    verify_template_output(
      "{% autoescape %}{% capture variable %}&{% endcapture %}{% endautoescape %}{{ variable }}",
      "&"
    )
  end

  it "can prevent escaping of assigned variables" do
    verify_template_output(
      '{% autoescape %}{% autoescape false %}{% assign variable = "&" %}{% endautoescape %}{{ variable }}{% endautoescape %}{{ variable }}',
      "&amp;&"
    )
  end

  describe "configuration options" do

    after(:each) { Liquid::Autoescape.reconfigure }

    context "with global mode enabled" do

      before(:each) do
        Liquid::Autoescape.configure do |config|
          config.global = true
        end
      end

      it "escapes variables outside of an autoescape block" do
        verify_template_output(
          "{{ variable }}",
          "&amp;",
          "variable" => "&"
        )
      end

      it "escapes variables in an autoescape block" do
        verify_template_output(
          "{% autoescape %}{{ variable }}{% endautoescape %}",
          "&amp;",
          "variable" => "&"
        )
      end

      it "respects escaping filters" do
        verify_template_output(
          "{{ variable | skip_escape }}{% autoescape %}{{ variable | skip_escape }}{% endautoescape %}",
          "&&",
          "variable" => "&"
        )
      end

      it "supports opting out of auto-escaping within a block" do
        verify_template_output(
          "{{ variable }}{% autoescape false %}{{ variable }}{{ variable }}{% endautoescape %}",
          "&amp;&&",
          "variable" => "&"
        )
      end

    end

    context "with custom exemptions" do

      let(:exemptions) do
        Module.new do
          def exemption(variable)
            variable.name == "module"
          end
        end
      end

      before(:each) do
        Liquid::Autoescape.configure do |config|
          config.exemptions.add { |variable| variable.name == "filter" }
          config.exemptions.import(exemptions)
        end
      end

      it "does not escape exempt variables" do
        verify_template_output(
          "{% autoescape %}{{ filter }} {{ module }} {{ other }}{% endautoescape %}",
          "<a> <b> &lt;i&gt;",
          "filter" => "<a>", "module" => "<b>", "other" => "<i>"
        )
      end

      it "can handle exemptions with lookup-style variable names" do
        Liquid::Autoescape.configure do |config|
          config.exemptions.add { |variable| variable.name == "root.one" }
          config.exemptions.add { |variable| variable.name == "trunk.branch.leaf" }
        end

        verify_template_output(
          "{% autoescape %}{{ root.one }} {{ root.two }} {{ trunk.branch.leaf }}{% endautoescape %}",
          "<a> &lt;b&gt; <i>",
          "root" => {"one" => "<a>", "two" => "<b>"},
          "trunk" => {"branch" => {"leaf" => "<i>"}}
        )
      end

      it "respects the default exemptions" do
        verify_template_output(
          "{% autoescape %}{{ filter | skip_escape }} {{ other }}{% endautoescape %}",
          "<a> &lt;b&gt;",
          "filter" => "<a>", "other" => "<b>"
        )
      end

    end

    context "with trusted filters" do

      before(:each) do
        Liquid::Autoescape.configure do |config|
          config.trusted_filters << :downcase
        end
      end

      it "does not forget the default escaping filters" do
        verify_template_output(
          "{% autoescape %}{{ one | skip_escape }} {{ two | escape }}{% endautoescape %}",
          "<a> &lt;b&gt;",
          "one" => "<a>", "two" => "<b>"
        )
      end

      it "exempts variables that use one of the trusted filters" do
        verify_template_output(
          "{% autoescape %}{{ variable | downcase }} {{ variable | capitalize }}{% endautoescape %}",
          "r&d R&amp;d",
          "variable" => "R&D"
        )
      end

    end

  end

end
